package it.unimi.dsi.law.triangrw;

/*		 
 * Copyright (C) 2011 Paolo Boldi and Marco Rosa 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */


/** An object that is able to compute the similarity between arcs of a graph. */
public interface ArcSimilarity  {
	/** Returns the similarity between two arcs; similarity is assumed to be a non-negative double, possibly (but not
	 *  mandatorily) not larger than 1. If either node pair is not an arc, the method is supposed to return 0.
	 * 
	 * @param from0 source of the first arc.
	 * @param to0 target of the first arc.
	 * @param from1 source of the second arc.
	 * @param to1 target of the second arc.
	 * @return similarity between arc <samp>(from0,to0)</samp> and arc <samp>(from1,to1)</samp> (or zero if either is not
	 * an arc). 
	 */
	public double sim( int from0, int to0, int from1, int to1 );
}

