package it.unimi.dsi.law.triangrw;

/*		 
 * Copyright (C) 2011 Paolo Boldi and Marco Rosa 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

import it.unimi.dsi.Util;
import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
import it.unimi.dsi.fastutil.ints.IntSet;
import it.unimi.dsi.fastutil.io.BinIO;
import it.unimi.dsi.fastutil.objects.Object2LongFunction;
import it.unimi.dsi.io.FastBufferedReader;
import it.unimi.dsi.lang.MutableString;
import it.unimi.dsi.logging.ProgressLogger;

import java.io.IOException;
import java.io.InputStreamReader;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.Parameter;
import com.martiansoftware.jsap.SimpleJSAP;
import com.martiansoftware.jsap.UnflaggedOption;

/** A tool that serves to count the number of occurrences of terms (for more information, please 
 *  use the "--help" option to the main).
 *
 */
public class CountTermFrequency  {
	private static final Logger LOGGER = Util.getLogger( CountTermFrequency.class ); 
	{
		Util.ensureLog4JIsConfigured();
	}


	@SuppressWarnings( "unchecked" )
	public static void main( String arg[] ) throws ParserConfigurationException, SAXException, IOException, ClassNotFoundException, JSAPException {
		final SimpleJSAP jsap = new SimpleJSAP( CountTermFrequency.class.getName(), "Reads (from stdin) lines of the form x\\ty\\tT, where x and y are integers and T is a space-separated sequence of terms, "+
				"and counts the number of lines where each term appears (terms are mapped to integers via the term map read as input).",
				new Parameter[] {
					new UnflaggedOption( "termMapFile", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, JSAP.NOT_GREEDY, "The minimal perfect term map." ),
					new UnflaggedOption( "countFile", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, JSAP.NOT_GREEDY, "The resulting count file (one integer per term)." )
		});

		JSAPResult jsapResult = jsap.parse( arg );
		if ( jsap.messagePrinted() ) return;
		

		Object2LongFunction<String> termMap = (Object2LongFunction<String>)BinIO.loadObject( jsapResult.getString( "termMapFile" ) );
		int n = termMap.size();
		int count[] = new int[ n ];
		IntSet termSet = new IntOpenHashSet();
		
		ProgressLogger pl = new ProgressLogger( LOGGER );
		pl.start( "Counting terms" );
		FastBufferedReader reader = new FastBufferedReader( new InputStreamReader( System.in ) );
		MutableString line = new MutableString();
		MutableString word = new MutableString();
		MutableString nonWord = new MutableString();
		MutableString first = new MutableString();
		MutableString second = new MutableString();
		
		while ( reader.readLine( line ) != null ) {
			FastBufferedReader lineReader = new FastBufferedReader( line );
			if ( !lineReader.next( first, nonWord ) || !lineReader.next( second, nonWord )) {
				System.err.println( "Malformed line: <" + line + ">" );
				return;
			}
			termSet.clear();
			while ( lineReader.next( word, nonWord ) ) {
				int term = (int)termMap.getLong( word );
				if ( term < 0 ) System.err.println( "Unknown term: <" + word + ">" );
				else termSet.add( term );
			}
			for ( int term: termSet ) count[ term ]++;
			pl.lightUpdate();
		}
		pl.done();
		BinIO.storeInts( count, jsapResult.getString( "countFile" ) );
	}
	
}
