package it.unimi.dsi.law.triangrw;

/*		 
 * Copyright (C) 2011 Paolo Boldi and Marco Rosa 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

import it.unimi.dsi.Util;
import it.unimi.dsi.fastutil.objects.ObjectOpenHashSet;
import it.unimi.dsi.fastutil.objects.ObjectSet;
import it.unimi.dsi.logging.ProgressLogger;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.Parameter;
import com.martiansoftware.jsap.SimpleJSAP;
import com.martiansoftware.jsap.UnflaggedOption;

/** Extract authors form the DBLP file (for more information, please 
 *  use the "--help" option to the main).
 */
public class ExtractAuthors extends DefaultHandler {
	public static final boolean DEBUG = false;
	private static final Logger LOGGER = Util.getLogger( ExtractAuthors.class ); 
	{
		Util.ensureLog4JIsConfigured();
	}

	private StringBuilder currentAuthor = new StringBuilder();
	private ObjectSet<String> authors = new ObjectOpenHashSet<String>();
	private boolean inAuthor = false;
	private final ProgressLogger pl;

	public ExtractAuthors( ProgressLogger progressLogger ) {
		this.pl = progressLogger;
	}

	@Override
	public void startElement( String uri, String localName, String qName, Attributes attributes ) {
		if ( qName.equals( "author" ) ) {
			inAuthor = true;
		} 
	}

	@Override
	public void endElement( String uri, String localName, String qName ) {
		if ( qName.equals( "author" ) && inAuthor ) {
			inAuthor = false;
			if ( authors.add( currentAuthor.toString() ) ) pl.lightUpdate();
			currentAuthor.setLength( 0 );
		}
	}

	@Override
	public void characters( char[] ch, int start, int length ) throws SAXException {
		if ( inAuthor ) currentAuthor.append( ch, start, length );
	}


	public static void main( String arg[] ) throws ParserConfigurationException, SAXException, IOException, JSAPException {
		final SimpleJSAP jsap = new SimpleJSAP( ExtractAuthors.class.getName(), "Parses a DBLP XML file and extracts the list of authors, one per line (with no duplicates).",
				new Parameter[] {
					new UnflaggedOption( "dblpXMLFile", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, JSAP.NOT_GREEDY, "The XML DBLP file to be parsed." )
		});

		JSAPResult jsapResult = jsap.parse( arg );
		if ( jsap.messagePrinted() ) return;
		

		String xmlFilename = jsapResult.getString( "dblpXMLFile" );
		SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
		ProgressLogger pl = new ProgressLogger( LOGGER ); 
		pl.start( "Parsing file" );
		ExtractAuthors extractAuthors = new ExtractAuthors( pl );
		parser.parse( xmlFilename, extractAuthors );
		pl.done();
		LOGGER.info( "Outputting author names" );
		for ( String author: extractAuthors.authors ) System.out.println( author );
		
	}
	
}
