package it.unimi.dsi.law.triangrw;

/*		 
 * Copyright (C) 2011 Paolo Boldi and Marco Rosa 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

import it.unimi.dsi.io.FastBufferedReader;
import it.unimi.dsi.lang.MutableString;
import it.unimi.dsi.logging.ProgressLogger;
import it.unimi.dsi.mg4j.index.snowball.PorterStemmer;

import java.io.IOException;
import java.io.InputStreamReader;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.Parameter;
import com.martiansoftware.jsap.SimpleJSAP;

/** Stem titles extracted form the DBLP file (for more information, please 
 *  use the "--help" option to the main).
 */
public class StemTitles  {
	public static void main( String arg[] ) throws ParserConfigurationException, SAXException, IOException, ClassNotFoundException, JSAPException {
		final SimpleJSAP jsap = new SimpleJSAP( StemTitles.class.getName(), "Reads (from stdin) lines of the form x\\ty\\tT, where x and y are integers and T is a text, "+
				"and outputs lines of the form x\\ty\\tT', where T' is obtained by breaking T into words, stemming them (via a Porter stemmer) and separating them by spaces.",
				new Parameter[] {
		});

		if ( jsap.messagePrinted() ) return;
		ProgressLogger pl = new ProgressLogger();
		
		FastBufferedReader reader = new FastBufferedReader( new InputStreamReader( System.in ) );
		MutableString line = new MutableString();
		MutableString word = new MutableString();
		MutableString nonWord = new MutableString();
		MutableString first = new MutableString();
		MutableString second = new MutableString();
		pl.start( "Stemming" );
		PorterStemmer stemmer = new PorterStemmer();
		
		while ( reader.readLine( line ) != null ) {
			FastBufferedReader lineReader = new FastBufferedReader( line );
			if ( !lineReader.next( first, nonWord ) || !lineReader.next( second, nonWord )) {
				System.err.println( "Malformed line: <" + line + ">" );
				return;
			}
			System.out.print( first + "\t" + second + "\t" );
			while ( lineReader.next( word, nonWord ) ) {
				stemmer.processTerm( word ); 
				System.out.print( word + " " );
			}
			System.out.println();
			pl.lightUpdate();
		}
		pl.done();
	}
	
}
